/*
 * BarSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Stepan Plachy
 */

#ifndef BAR_SYMBOL_H_
#define BAR_SYMBOL_H_

#include "Symbol.h"

namespace alphabet {

/**
 * Represents bar symbol for tree linearization.
 */
class BarSymbol : public std::acceptor<BarSymbol, VisitableSymbolBase, std::acceptor<BarSymbol, alib::VisitableObjectBase, SymbolBase> > {
public:
	/**
	 * Creates a bar symbol.
	 * @param symbol name of the symbol
	 */
	explicit BarSymbol();

	virtual SymbolBase* clone() const;
	virtual SymbolBase* plunder() &&;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const BarSymbol& other) const;

	virtual void operator>>(std::ostream& out) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<BarSymbol>();
	}

	static BarSymbol BAR;
};

} /* namespace alphabet */

namespace std {

template<>
struct compare<alphabet::BarSymbol> {
	int operator()(const alphabet::BarSymbol& first, const alphabet::BarSymbol& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* BAR_SYMBOL_H_ */
