/*
 * LabeledSymbol.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Martin Zak
 */

#ifndef LABELED_SYMBOL_H_
#define LABELED_SYMBOL_H_

#include "../label/Label.h"
#include "SymbolBase.h"
#include <ostream>

namespace alphabet {

/**
 * Represents symbol in an alphabet.
 */
class LabeledSymbol : public std::acceptor<LabeledSymbol, VisitableSymbolBase, std::acceptor<LabeledSymbol, alib::VisitableObjectBase, SymbolBase> > {
protected:
	label::Label label;
	
public:
	explicit LabeledSymbol(int number);
	explicit LabeledSymbol(char character);
	explicit LabeledSymbol(const std::string& label);

	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit LabeledSymbol(const label::Label& label);
	explicit LabeledSymbol(label::Label&& label);

	virtual SymbolBase* clone() const;
	virtual SymbolBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const label::Label& getLabel() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const LabeledSymbol& other) const;

	virtual void operator>>(std::ostream& out) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<LabeledSymbol>();
	}
};

} /* namespace alphabet */

namespace std {

template<>
struct compare<alphabet::LabeledSymbol> {
	int operator()(const alphabet::LabeledSymbol& first, const alphabet::LabeledSymbol& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* LABELED_SYMBOL_H_ */
