/*
 * RankedBarSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef RANKED_BAR_SYMBOL_H_
#define RANKED_BAR_SYMBOL_H_

#include "Symbol.h"
#include "../primitive/Integer.h"

namespace alphabet {

/**
 * Represents rankedBar symbol for tree linearization.
 */
class RankedBarSymbol : public std::acceptor<RankedBarSymbol, VisitableSymbolBase, std::acceptor<RankedBarSymbol, alib::VisitableObjectBase, SymbolBase> > {
	primitive::Integer rank;
public:
	/**
	 * Creates a rankedBar symbol.
	 * @param symbol name of the symbol
	 */
	explicit RankedBarSymbol(int rank);
	explicit RankedBarSymbol(const primitive::Integer& rank);

	virtual SymbolBase* clone() const;
	virtual SymbolBase* plunder() &&;

	/**
	 * @return rank of the symbol
	 */
	const primitive::Integer& getRank() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const RankedBarSymbol& other) const;

	virtual void operator>>(std::ostream& out) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<RankedBarSymbol>();
	}
};

} /* namespace alphabet */

namespace std {

template<>
struct compare<alphabet::RankedBarSymbol> {
	int operator()(const alphabet::RankedBarSymbol& first, const alphabet::RankedBarSymbol& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* RANKED_BAR_SYMBOL_H_ */
