/*
 * RankedSymbol.cpp
 *
 *  Created on: Apr 10, 2013
 *      Author: Stepan Plachy
 */

#include "RankedSymbol.h"

namespace alphabet {

RankedSymbol::RankedSymbol(int number, int rank) : label(label::labelFrom(number)), rank(primitive::Integer(rank)) {

}

RankedSymbol::RankedSymbol(char character, int rank) : label(label::labelFrom(character)), rank(primitive::Integer(rank)) {

}

RankedSymbol::RankedSymbol(const std::string& label, int rank) : label(label::labelFrom(label)), rank(primitive::Integer(rank)) {

}

RankedSymbol::RankedSymbol(const label::Label& label, const primitive::Integer& rank) : label(label), rank(rank) {

}

RankedSymbol::RankedSymbol(label::Label&& label, primitive::Integer&& rank) : label(std::move(label)), rank(std::move(rank)) {

}

SymbolBase* RankedSymbol::clone() const {
	return new RankedSymbol(*this);
}

SymbolBase* RankedSymbol::plunder() && {
	return new RankedSymbol(std::move(*this));
}

const label::Label& RankedSymbol::getLabel() const {
	return label;
}

const primitive::Integer& RankedSymbol::getRank() const {
	return rank;
}

int RankedSymbol::compare(const RankedSymbol& other) const {
	int res = this->label.getData().compare(other.label.getData());
	if(res == 0) res = this->rank.compare(other.rank);
	return res;
}

void RankedSymbol::operator>>(std::ostream& out) const {
	out << "(RankedSymbol " << label << " #" << rank << ")";
}

RankedSymbol::operator std::string () const {
	return (std::string) label + "_" + (std::string) rank;
}

} /* namespace alphabet */

