/*
 * SymbolBase.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_BASE_H_
#define SYMBOL_BASE_H_

#include "../common/base.hpp"
#include "../object/ObjectBase.h"

namespace alphabet {

class SymbolBase;

typedef std::acceptor_base<SymbolBase,
			EndSymbol, LabeledSymbol, BlankSymbol, BottomOfTheStackSymbol, RankedSymbol, BarSymbol, RankedBarSymbol, SubtreeWildcardSymbol, SymbolSetSymbol, SymbolPairSymbol, UniqueSymbol, StartSymbol
	> VisitableSymbolBase;

/**
 * Represents symbol in an alphabet.
 */
class SymbolBase : public alib::ObjectBase, public VisitableSymbolBase {
public:
	using VisitableSymbolBase::Accept;
	using alib::VisitableObjectBase::Accept;

	virtual SymbolBase* clone() const = 0;
	virtual SymbolBase* plunder() && = 0;

};

} /* namespace alphabet */

namespace std {

template<>
struct compare<alphabet::SymbolBase> {
	int operator()(const alphabet::SymbolBase& first, const alphabet::SymbolBase& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* SYMBOL_BASE_H_ */
