/*
 * SymbolFromStringLexer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "SymbolFromStringLexer.h"
#include "../label/LabelFromStringLexer.h"

namespace alphabet {

SymbolFromStringLexer::Token SymbolFromStringLexer::next(std::istream& in) const {
	SymbolFromStringLexer::Token token;
	token.type = TokenType::ERROR;
	token.value = "";
	token.raw = "";
	char character;

L0:
	character = in.get();
	if(in.eof()) {
		token.type = TokenType::TEOF;
		return token;
	} else if(character == ' ' || character == '\n' || character == '\t') {
		token.raw += character;
		goto L0;
	} else if(character == '#') {
		token.raw += character;
		goto L1;
	} else {
		in.putback(character);
		putback(in, std::move(token));
		token.type = TokenType::ERROR;
		return token;
	}
L1:
	character = in.get();
	if(in.eof()) {
		token.type = TokenType::TEOF;
		return token;
	} else if(character == 'B') {
		token.type = TokenType::BLANK;
		token.raw += character;
		return token;
	} else if(character == 'Z') {
		token.type = TokenType::BOTTOM;
		token.raw += character;
		return token;
	} else if(character == 'E') {
		token.type = TokenType::END;
		token.raw += character;
		return token;
	} else if(character == 'S') {
		token.type = TokenType::START;
		token.raw += character;
		return token;
	} else {
		in.putback(character);
		putback(in, std::move(token));
		token.type = TokenType::ERROR;
		return token;
	}
}

void SymbolFromStringLexer::putback(std::istream& in, SymbolFromStringLexer::Token token) const {
	while(!token.raw.empty()) {
		in.putback(token.raw.back());
		token.raw.pop_back();
	}
	in.clear();
}

} /* namespace alphabet */
