/*
 * SymbolPairSymbol.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_PAIR_SYMBOL_H_
#define SYMBOL_PAIR_SYMBOL_H_

#include <string>
#include <ostream>

#include "Symbol.h"
#include "SymbolBase.h"
#include <set>

namespace alphabet {

/**
 * Represents symbol in an alphabet.
 */
class SymbolPairSymbol : public std::acceptor<SymbolPairSymbol, VisitableSymbolBase, std::acceptor<SymbolPairSymbol, alib::VisitableObjectBase, SymbolBase> > {
protected:
	std::pair<Symbol, Symbol> symbol;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit SymbolPairSymbol(const std::pair<Symbol, Symbol>& symbol);

	explicit SymbolPairSymbol(std::pair<Symbol, Symbol>&& symbol);

	virtual SymbolBase* clone() const;

	virtual SymbolBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const std::pair<Symbol, Symbol>& getData() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const SymbolPairSymbol& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<SymbolPairSymbol>();
	}
};

} /* namespace alphabet */

namespace std {

template<>
struct compare<alphabet::SymbolPairSymbol> {
	int operator()(const alphabet::SymbolPairSymbol& first, const alphabet::SymbolPairSymbol& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* SYMBOL_PAIR_SYMBOL_H_ */

