/*
 * UniqueSymbol.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_ID_SYMBOL_H_
#define SYMBOL_ID_SYMBOL_H_

#include <string>
#include <ostream>

#include "Symbol.h"
#include "SymbolBase.h"
#include <set>
#include "../primitive/Integer.h"

namespace alphabet {

/**
 * Represents symbol in an alphabet.
 */
class UniqueSymbol : public std::acceptor<UniqueSymbol, VisitableSymbolBase, std::acceptor<UniqueSymbol, alib::VisitableObjectBase, SymbolBase> > {
protected:
	Symbol symbol;
	primitive::Integer id;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit UniqueSymbol(const Symbol& symbol, const primitive::Integer& id);

	virtual SymbolBase* clone() const;

	virtual SymbolBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const Symbol& getSymbol() const;

	const primitive::Integer& getId() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const UniqueSymbol& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<UniqueSymbol>();
	}
};

} /* namespace alphabet */

namespace std {

template<>
struct compare<alphabet::UniqueSymbol> {
	int operator()(const alphabet::UniqueSymbol& first, const alphabet::UniqueSymbol& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* SYMBOL_ID_SYMBOL_H_ */

