/*
 * CallReturnLocalInputAlphabet.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "CallReturnLocalInputAlphabet.h"

#include <algorithm>
#include "../AutomatonException.h"

namespace automaton {

const std::set<alphabet::Symbol>& CallReturnLocalInputAlphabet::getCallInputAlphabet() const {
	return callInputAlphabet;
}

bool CallReturnLocalInputAlphabet::addCallInputSymbol(const alphabet::Symbol& symbol) {
	if(returnInputAlphabet.count(symbol))
		throw exception::AlibException("Input symbol " + (std::string) symbol + " already in return alphabet");
	if(localInputAlphabet.count(symbol))
		throw exception::AlibException("Input symbol " + (std::string) symbol + " already in local alphabet");
	return callInputAlphabet.insert(symbol).second;
}

void CallReturnLocalInputAlphabet::setCallInputAlphabet(const std::set<alphabet::Symbol>& newSymbols) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(callInputAlphabet.begin(), callInputAlphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	std::set<alphabet::Symbol> added;
	std::set_difference(newSymbols.begin(), newSymbols.end(), callInputAlphabet.begin(), callInputAlphabet.end(), std::inserter(added, added.end()));
	
	for(const alphabet::Symbol& removedSymbol : removed) {
		removeInputSymbol(removedSymbol);
	}

	for(const alphabet::Symbol& addedSymbol : added) {
		addCallInputSymbol(addedSymbol);
	}
}

const std::set<alphabet::Symbol>& CallReturnLocalInputAlphabet::getReturnInputAlphabet() const {
	return returnInputAlphabet;
}

bool CallReturnLocalInputAlphabet::addReturnInputSymbol(const alphabet::Symbol& symbol) {
	if(localInputAlphabet.count(symbol))
		throw exception::AlibException("Input symbol " + (std::string) symbol + " already in local alphabet");
	if(callInputAlphabet.count(symbol))
		throw exception::AlibException("Input symbol " + (std::string) symbol + " already in call alphabet");
	return returnInputAlphabet.insert(symbol).second;
}

void CallReturnLocalInputAlphabet::setReturnInputAlphabet(const std::set<alphabet::Symbol>& newSymbols) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(returnInputAlphabet.begin(), returnInputAlphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	std::set<alphabet::Symbol> added;
	std::set_difference(newSymbols.begin(), newSymbols.end(), returnInputAlphabet.begin(), returnInputAlphabet.end(), std::inserter(added, added.end()));
	
	for(const alphabet::Symbol& removedSymbol : removed) {
		removeInputSymbol(removedSymbol);
	}

	for(const alphabet::Symbol& addedSymbol : added) {
		addReturnInputSymbol(addedSymbol);
	}
}

const std::set<alphabet::Symbol>& CallReturnLocalInputAlphabet::getLocalInputAlphabet() const {
	return localInputAlphabet;
}

bool CallReturnLocalInputAlphabet::addLocalInputSymbol(const alphabet::Symbol& symbol) {
	if(callInputAlphabet.count(symbol))
		throw exception::AlibException("Input symbol " + (std::string) symbol + " already in call alphabet");
	if(returnInputAlphabet.count(symbol))
		throw exception::AlibException("Input symbol " + (std::string) symbol + " already in return alphabet");
	return localInputAlphabet.insert(symbol).second;
}

void CallReturnLocalInputAlphabet::setLocalInputAlphabet(const std::set<alphabet::Symbol>& newSymbols) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(localInputAlphabet.begin(), localInputAlphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	std::set<alphabet::Symbol> added;
	std::set_difference(newSymbols.begin(), newSymbols.end(), localInputAlphabet.begin(), localInputAlphabet.end(), std::inserter(added, added.end()));
	
	for(const alphabet::Symbol& removedSymbol : removed) {
		removeInputSymbol(removedSymbol);
	}

	for(const alphabet::Symbol& addedSymbol : added) {
		addLocalInputSymbol(addedSymbol);
	}
}

} /* namespace automaton */

