/*
 * Base.h
 *
 *  Created on: Apr 05, 2014
 *      Author: Jan Travnicek
 */

#ifndef BASE_H_
#define BASE_H_

#include <typeinfo>
#include <ostream>
#include <stdexcept>

namespace alib {

template<int ID>
class base_base {
public:
	virtual int selfTypeId() const = 0;

	virtual ~base_base() noexcept {

	}
};

template<int ID, typename... Types>
class base_helper;

template<int ID, typename T>
class base_helper<ID, T> : public base_base<ID + 1> {
public:
	template<typename R, typename std::enable_if < std::is_same< R, T >::value >::type* = nullptr >
	static int typeId() {
		return ID;
	}

	virtual int compare(const T &) const {
		if(this->selfTypeId() < typeId<T>())
			return -1;
		else if(this->selfTypeId() > typeId<T>())
			return 1;
		else
			throw std::logic_error("Should not be reached");
	}
};

template<int ID, typename T, typename... Types>
class base_helper<ID, T, Types...> : public base_helper<ID + 1, Types...> {
public:
	using base_helper<ID + 1, Types...>::compare;
	using base_helper<ID + 1, Types...>::typeId;

	template<typename R, typename std::enable_if < std::is_same< R, T >::value >::type* = nullptr >
	static int typeId() {
		return ID;
	}

	virtual int compare(const T &) const {
		if(this->selfTypeId() < typeId<T>())
			return -1;
		else if(this->selfTypeId() > typeId<T>())
			return 1;
		else
			throw std::logic_error("Should not be reached");
	}
};

template<typename T, typename... Types>
class base : public base_helper<1, Types...> {
public:
	using base_helper<1, Types...>::compare;
	using base_helper<1, Types...>::typeId;

	virtual T* clone() const = 0;

	virtual T* plunder() && = 0;

	bool operator!=(const T& other) const {
		return !(*this == other);
	}

	bool operator==(const T& other) const {
		return this->compare(other) == 0;
	}

	bool operator<(const T& other) const {
		return this->compare(other) < 0;
	}

	bool operator>(const T& other) const {
		return this->compare(other) > 0;
	}

	virtual int compare(const T& other) const = 0;

	friend std::ostream& operator<<(std::ostream& os, const T& instance) {
		instance >> os;
		return os;
	}

	virtual void operator>>(std::ostream&) const = 0;

	virtual operator std::string () const = 0;
};

} /* namespace alib */

#endif /* BASE_H_ */
