/*
 * ContainerBase.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef CONTAINER_BASE_H_
#define CONTAINER_BASE_H_

#include "../common/base.hpp"
#include "../object/ObjectBase.h"

namespace container {

class ContainerBase;

typedef std::acceptor_base<ContainerBase,
			container::ObjectsSet,
			container::ObjectsVector,
			container::ObjectsPair,
			container::ObjectsMap
	> VisitableContainerBase;

/**
 * Represents container in a hierarchy.
 */
class ContainerBase : public alib::ObjectBase, public VisitableContainerBase {
public:
	using VisitableContainerBase::Accept;
	using alib::VisitableObjectBase::Accept;

	virtual ContainerBase* clone() const = 0;
	virtual ContainerBase* plunder() && = 0;

};

} /* namespace container */

namespace std {

template<>
struct compare<container::ContainerBase> {
	int operator()(const container::ContainerBase& first, const container::ContainerBase& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* CONTAINER_BASE_H_ */
