/*
 * Pair.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_PAIR_H_
#define OBJECTS_PAIR_H_

#include <map>
#include <string>

#include "../object/Object.h"
#include "ContainerBase.h"

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
class ObjectsPair : public std::pair<alib::Object, alib::Object>, public std::acceptor<ObjectsPair, VisitableContainerBase, std::acceptor<ObjectsPair, alib::VisitableObjectBase, ContainerBase> > {
public:
	explicit ObjectsPair(const alib::Object& first, const alib::Object& second);

	virtual ContainerBase* clone() const;
	
	virtual ContainerBase* plunder() &&;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const ObjectsPair& other) const;

	virtual void operator>>(std::ostream& os) const;

	virtual operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<ObjectsPair>();
	}
};

} /* namespace container */

namespace std {

template<>
struct compare<container::ObjectsPair> {
	int operator()(const container::ObjectsPair& first, const container::ObjectsPair& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* OBJECTS_PAIR_H_ */
