/*
 * AlibException.h
 *
 * Created on: Apr 1, 2013
 * Author: Martin Zak
 */

#ifndef ALIB_EXCEPTION_H_
#define ALIB_EXCEPTION_H_

#include <exception>
#include <string>

#include "../object/ObjectBase.h"

namespace exception {

/**
 * Basic exception from which are derived all other exceptions.
 * Contains reason why the exception occured.
 */
class AlibException : public std::exception, public std::acceptor<AlibException, alib::VisitableObjectBase, alib::ObjectBase> {
protected:
	std::string cause;
	std::string backtrace;
	std::string whatMessage;

public:
	explicit AlibException ( );

	explicit AlibException ( const std::string & cause );

	explicit AlibException ( const std::string & cause, const std::string& backtrace );

	virtual ~AlibException ( ) noexcept;

	virtual alib::ObjectBase* clone() const;
	
	virtual alib::ObjectBase* plunder() &&;

	/**
	 * @return reason why the exception occured
	 */
	const char * what ( ) const noexcept;

	/**
	 * @return reason why the exception occured
	 */
	const std::string & getCause ( ) const;

	/**
	 * @return reason why the exception occured
	 */
	const std::string & getBacktrace ( ) const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const AlibException& other) const;

	virtual void operator>>(std::ostream& os) const;

	virtual operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<AlibException>();
	}
};

} /* namespace exception */

namespace std {

template<>
struct compare<::exception::AlibException> {
	int operator()(const ::exception::AlibException& first, const ::exception::AlibException& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* ALIB_EXCEPTION_H_ */
