/*
 * StringDataFactory.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef STRING_DATA_FACTORY
#define STRING_DATA_FACTORY

#include <set>
#include "../exception/AlibException.h"
#include "../StringApi.hpp"
#include <cctype>

namespace alib {

class StringDataFactory {
public:

	template<class T>
	static T fromFile(const std::string& filename) {
		std::ifstream fileStream(filename);
		return fromStream<T>(fileStream);
	}

	template<class T>
	static T fromString(const std::string& str) {
		std::stringstream stringStream(str);
		return fromStream<T>(stringStream);
	}

	template<class T>
	static T fromStdin() {
		return fromStream<T>(std::cin);
	}

	template<class T>
	static T fromStream(std::istream& in) {
		if(in.peek() == EOF) throw exception::AlibException("Empty stream");

		T res = alib::stringApi<T>::parse(in);

		while(isspace(in.peek())) in.get();
		if(in.peek() != EOF) throw exception::AlibException("Unexpected characters at the end of the stream");
		return res;
	}

	template<class T>
	static void toFile(const T& data, const std::string& filename) {
		std::ofstream fileStream(filename);
		toStream<T>(data, fileStream);
	}

	template<class T>
	static std::string toString(const T& data) {
		std::stringstream stringStream;
		toStream<T>(data, stringStream);
		return stringStream.str();
	}

	template<class T>
	static void toStdout(const T& data) {
		toStream<T>(data, std::cout);
	}

	template<class T>
	static void toStream(const T& data, std::ostream& out) {
		alib::stringApi<T>::compose(out, data);
	}
};

} /* namespace alib */

#endif /* STRING_DATA_FACTORY */
