/*
 * GrammarFromXMLParser.h
 *
 *  Created on: Oct 12, 2013
 *      Author: Martin Zak
 */

#ifndef GRAMMAR_FROM_XML_PARSER_H_
#define GRAMMAR_FROM_XML_PARSER_H_

#include "../sax/FromXMLParserHelper.h"
#include "Grammar.h"
#include "GrammarFeatures.h"
#include "Regular/LeftLG.h"
#include "Regular/LeftRG.h"
#include "Regular/RightLG.h"
#include "Regular/RightRG.h"
#include "ContextFree/LG.h"
#include "ContextFree/CFG.h"
#include "ContextFree/EpsilonFreeCFG.h"
#include "ContextFree/CNF.h"
#include "ContextFree/GNF.h"
#include "ContextSensitive/CSG.h"
#include "ContextSensitive/NonContractingGrammar.h"
#include "Unrestricted/ContextPreservingUnrestrictedGrammar.h"
#include "Unrestricted/UnrestrictedGrammar.h"

#include <list>
#include <set>
#include "../std/variant.hpp"
#include "../sax/Token.h"

namespace alib {

template<typename T>
struct xmlApi;

} /* namespace alib */

namespace grammar {

/**
 * Parser used to get general FSM or EpsilonNFA, NFA, DFA from XML parsed into list of Tokens.
 */
class GrammarFromXMLParser : public sax::FromXMLParserHelper {
	std::set<alphabet::Symbol> parseNonterminalAlphabet(std::list<sax::Token> &input) const;
	std::set<alphabet::Symbol> parseTerminalAlphabet(std::list<sax::Token> &input) const;
	alphabet::Symbol parseInitialSymbol(std::list<sax::Token> &input) const;
	bool parseGeneratesEpsilon(std::list<sax::Token> &input) const;

	std::vector<alphabet::Symbol> parseRuleLContext(std::list<sax::Token>& input) const;
	std::vector<alphabet::Symbol> parseRuleLHS(std::list<sax::Token>& input) const;
	std::vector<alphabet::Symbol> parseRuleRContext(std::list<sax::Token>& input) const;
	alphabet::Symbol parseRuleSingleSymbolLHS(std::list<sax::Token>& input) const;
	std::pair<alphabet::Symbol, std::vector<alphabet::Symbol>> parseRuleGNFRHS(std::list<sax::Token>& input) const;
	std::vector<alphabet::Symbol> parseRuleRHS(std::list<sax::Token>& input) const;
	std::variant<alphabet::Symbol, std::pair<alphabet::Symbol, alphabet::Symbol>> parseRuleOneOrTwoSymbolsRHS(std::list<sax::Token>& input) const;

	template<class T>
	void parseRules(std::list<sax::Token> &input, T& grammar) const;

	void parseRule(std::list<sax::Token>& input, UnrestrictedGrammar& grammar) const;
	void parseRule(std::list<sax::Token>& input, ContextPreservingUnrestrictedGrammar& grammar) const;
	void parseRule(std::list<sax::Token>& input, NonContractingGrammar& grammar) const;
	void parseRule(std::list<sax::Token>& input, CSG& grammar) const;
	void parseRule(std::list<sax::Token>& input, GNF& grammar) const;
	void parseRule(std::list<sax::Token>& input, CNF& grammar) const;
	void parseRule(std::list<sax::Token>& input, EpsilonFreeCFG& grammar) const;
	void parseRule(std::list<sax::Token>& input, CFG& grammar) const;
	void parseRule(std::list<sax::Token>& input, LG& grammar) const;
	void parseRule(std::list<sax::Token>& input, RightRG& grammar) const;
	void parseRule(std::list<sax::Token>& input, RightLG& grammar) const;
	void parseRule(std::list<sax::Token>& input, LeftRG& grammar) const;
	void parseRule(std::list<sax::Token>& input, LeftLG& grammar) const;

	Grammar parseGrammar(std::list<sax::Token>& input) const;
	Grammar parseGrammar(std::list<sax::Token>& input, const std::set<FEATURES>& features) const;

	UnrestrictedGrammar parseUnrestrictedGrammar(std::list<sax::Token>& input) const;
	ContextPreservingUnrestrictedGrammar parseContextPreservingUnrestrictedGrammar(std::list<sax::Token>& input) const;
	NonContractingGrammar parseNonContractingGrammar(std::list<sax::Token>& input) const;
	CSG parseCSG(std::list<sax::Token>& input) const;
	GNF parseGNF(std::list<sax::Token>& input) const;
	CNF parseCNF(std::list<sax::Token>& input) const;
	EpsilonFreeCFG parseEpsilonFreeCFG(std::list<sax::Token>& input) const;
	CFG parseCFG(std::list<sax::Token>& input) const;
	LG parseLG(std::list<sax::Token>& input) const;
	RightRG parseRightRG(std::list<sax::Token>& input) const;
	RightLG parseRightLG(std::list<sax::Token>& input) const;
	LeftRG parseLeftRG(std::list<sax::Token>& input) const;
	LeftLG parseLeftLG(std::list<sax::Token>& input) const;

	template<typename T> friend class alib::xmlApi;
public:
	bool first(const std::list<sax::Token>& input) const;
};

} /* namespace grammar */

#endif /* GRAMMAR_FROM_XML_PARSER_H_ */

