/*
 * UnrestrictedGrammar.h
 *
 *  Created on: Nov 17, 2013
 *      Author: Jan Travnicek
 */

#ifndef UNRESTRICTED_GRAMMAR_H_
#define UNRESTRICTED_GRAMMAR_H_

#include "../GrammarBase.h"
#include <map>
#include <vector>
#include "../common/TerminalNonterminalAlphabetInitialSymbol.h"

namespace grammar {

/**
 * Unrestricted grammar. Type 0 in Chomsky hierarchy. Produces recursively enumerable language.
 */
class UnrestrictedGrammar : public std::acceptor<UnrestrictedGrammar, VisitableGrammarBase, std::acceptor<UnrestrictedGrammar, alib::VisitableObjectBase, GrammarBase> >, public TerminalNonterminalAlphabetInitialSymbol {
	std::map<std::vector<alphabet::Symbol>, std::set<std::vector<alphabet::Symbol>>> rules;

public:
	explicit UnrestrictedGrammar(const alphabet::Symbol& initialSymbol);

	explicit UnrestrictedGrammar(const std::set<alphabet::Symbol>& nonTerminalSymbols, const std::set<alphabet::Symbol>& terminalSymbols, const alphabet::Symbol& initialSymbol);

	virtual GrammarBase* clone() const;

	virtual GrammarBase* plunder() &&;

	bool addRule(const std::vector<alphabet::Symbol>& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);

	const std::map<std::vector<alphabet::Symbol>, std::set<std::vector<alphabet::Symbol>>> & getRules() const;

	bool removeRule(const std::vector<alphabet::Symbol>& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);

	bool removeTerminalSymbol(const alphabet::Symbol& symbol);

	bool removeNonterminalSymbol(const alphabet::Symbol& symbol);

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const UnrestrictedGrammar& other) const;

	virtual void operator>>(std::ostream& os) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<UnrestrictedGrammar>();
	}
};

} /* namespace grammar */

namespace std {

template<>
struct compare<grammar::UnrestrictedGrammar> {
	int operator()(const grammar::UnrestrictedGrammar& first, const grammar::UnrestrictedGrammar& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* UNRESTRICTED_GRAMMAR_H_ */
