/*
 * LabelPairLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef LABEL_PAIR_LABEL_H_
#define LABEL_PAIR_LABEL_H_

#include <string>
#include <ostream>

#include "Label.h"
#include "LabelBase.h"
#include <set>

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class LabelPairLabel : public std::acceptor<LabelPairLabel, VisitableLabelBase, std::acceptor<LabelPairLabel, alib::VisitableObjectBase, LabelBase> > {
protected:
	std::pair<Label, Label> label;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit LabelPairLabel(const std::pair<Label, Label>& label);

	explicit LabelPairLabel(std::pair<Label, Label>&& label);

	virtual LabelBase* clone() const;

	virtual LabelBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const std::pair<Label, Label>& getData() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const LabelPairLabel& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<LabelPairLabel>();
	}
};

} /* namespace label */

namespace std {

template<>
struct compare<label::LabelPairLabel> {
	int operator()(const label::LabelPairLabel& first, const label::LabelPairLabel& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* LABEL_PAIR_LABEL_H_ */

