/*
 * ObjectLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef OBJECT_LABEL_H_
#define OBJECT_LABEL_H_

#include <ostream>

#include "LabelBase.h"
#include "../object/Object.h"

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class ObjectLabel : public std::acceptor<ObjectLabel, VisitableLabelBase, std::acceptor<ObjectLabel, alib::VisitableObjectBase, LabelBase> > {
protected:
	alib::Object label;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit ObjectLabel(const alib::Object& label);

	explicit ObjectLabel(alib::Object&& label);

	virtual LabelBase* clone() const;

	virtual LabelBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const alib::Object& getData() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const ObjectLabel& other) const;
	
	virtual void operator>>(std::ostream&) const;

	virtual operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<ObjectLabel>();
	}
};

} /* namespace label */

namespace std {

template<>
struct compare<label::ObjectLabel> {
	int operator()(const label::ObjectLabel& first, const label::ObjectLabel& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* OBJECT_LABEL_H_ */
