/*
 * Bool.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_BOOL_H_
#define PRIMITIVE_BOOL_H_

#include <ostream>

#include "PrimitiveBase.h"

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class Bool : public std::acceptor<Bool, VisitablePrimitiveBase, std::acceptor<Bool, alib::VisitableObjectBase, PrimitiveBase> > {
protected:
	bool data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit Bool(bool data);

	virtual PrimitiveBase* clone() const;

	virtual PrimitiveBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	bool getData() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const Bool& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<Bool>();
	}
};

} /* namespace primitive */

namespace std {

template<>
struct compare<primitive::Bool> {
	int operator()(const primitive::Bool& first, const primitive::Bool& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* PRIMITIVE_BOOL_H_ */
