/*
 * Character.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_CHARACTER_H_
#define PRIMITIVE_CHARACTER_H_

#include <string>
#include <ostream>

#include "PrimitiveBase.h"

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class Character : public std::acceptor<Character, VisitablePrimitiveBase, std::acceptor<Character, alib::VisitableObjectBase, PrimitiveBase> > {
protected:
	char data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit Character(char data);

	virtual PrimitiveBase* clone() const;

	virtual PrimitiveBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	char getData() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const Character& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<Character>();
	}
};

} /* namespace primitive */

namespace std {

template<>
struct compare<primitive::Character> {
	int operator()(const primitive::Character& first, const primitive::Character& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* PRIMITIVE_CHARACTER_H_ */

