/*
 * PrimitiveBase.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_BASE_H_
#define PRIMITIVE_BASE_H_

#include "../common/base.hpp"
#include "../object/ObjectBase.h"

namespace primitive {

class PrimitiveBase;

typedef std::acceptor_base<PrimitiveBase,
			primitive::String, primitive::Integer, primitive::Character, primitive::Unsigned, primitive::Bool
	> VisitablePrimitiveBase;

/**
 * Abstract base class for all automata.
 */
class PrimitiveBase : public alib::ObjectBase, public VisitablePrimitiveBase {
public:
	using VisitablePrimitiveBase::Accept;
	using alib::VisitableObjectBase::Accept;

	virtual PrimitiveBase* clone() const = 0;
	virtual PrimitiveBase* plunder() && = 0;

};

} /* namespace primitive */

namespace std {

template<>
struct compare<primitive::PrimitiveBase> {
	int operator()(const primitive::PrimitiveBase& first, const primitive::PrimitiveBase& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* PRIMITIVE_BASE_H_ */

