/*
 * PrimitiveFromXMLParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_FROM_XML_PARSER_H_
#define PRIMITIVE_FROM_XML_PARSER_H_

#include "../sax/FromXMLParserHelper.h"
#include <set>
#include "Primitive.h"
#include "PrimitiveFeatures.h"
#include "String.h"
#include "Integer.h"
#include "Character.h"
#include "Unsigned.h"
#include "Bool.h"
#include "../sax/Token.h"

namespace alib {

template<typename T>
struct xmlApi;

} /* namespace alib */

namespace primitive {

/**
 * Parser used to transform sequence of xml tokens to internal representation of Primitive.
 */
class PrimitiveFromXMLParser : public sax::FromXMLParserHelper {
	Primitive parsePrimitive(std::list<sax::Token>& input, const std::set<FEATURES>&) const;
	Primitive parsePrimitive(std::list<sax::Token>& input) const;

	Integer parseInteger(std::list<sax::Token>& input) const;
	Character parseCharacter(std::list<sax::Token>& input) const;
	String parseString(std::list<sax::Token>& input) const;
	Unsigned parseUnsigned(std::list<sax::Token>& input) const;
	Bool parseBool(std::list<sax::Token>& input) const;

	int parseIntegerRaw(std::list<sax::Token>& input) const;
	char parseCharacterRaw(std::list<sax::Token>& input) const;
	std::string parseStringRaw(std::list<sax::Token>& input) const;
	unsigned parseUnsignedRaw(std::list<sax::Token>& input) const;
	bool parseBoolRaw(std::list<sax::Token>& input) const;

	template<typename T> friend class alib::xmlApi;
public:
	bool first(const std::list<sax::Token>& input) const;
};

} /* namespace primitive */

#endif /* PRIMITIVE_FROM_XML_PARSER_H_ */
