/*
 * PrimitiveToXMLComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "PrimitiveToXMLComposer.h"
#include "Integer.h"
#include "String.h"
#include "Character.h"
#include "../std/itos.h"
#include "../std/utos.h"

#include "../XmlApi.hpp"

namespace primitive {

void PrimitiveToXMLComposer::compose(std::list<sax::Token>& out, const Primitive& primitive) const {
	primitive.getData().Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
}

void PrimitiveToXMLComposer::compose(std::list<sax::Token>& out, const PrimitiveBase& primitive) const {
	primitive.Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
}

void PrimitiveToXMLComposer::compose(std::list<sax::Token>& out, const Integer& primitive) const {
	out.push_back(sax::Token(alib::Names::PRIMITIVE_INTEGER, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(std::itos(primitive.getData()), sax::Token::TokenType::CHARACTER));
	out.push_back(sax::Token(alib::Names::PRIMITIVE_INTEGER, sax::Token::TokenType::END_ELEMENT));
}

void PrimitiveToXMLComposer::compose(std::list<sax::Token>& out, const Character& primitive) const {
	out.push_back(sax::Token(alib::Names::PRIMITIVE_CHARACTER, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(std::string(1, primitive.getData()), sax::Token::TokenType::CHARACTER));
	out.push_back(sax::Token(alib::Names::PRIMITIVE_CHARACTER, sax::Token::TokenType::END_ELEMENT));
}

void PrimitiveToXMLComposer::compose(std::list<sax::Token>& out, const Unsigned& primitive) const {
	out.push_back(sax::Token(alib::Names::PRIMITIVE_UNSIGNED, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(std::utos(primitive.getData()), sax::Token::TokenType::CHARACTER));
	out.push_back(sax::Token(alib::Names::PRIMITIVE_UNSIGNED, sax::Token::TokenType::END_ELEMENT));
}

void PrimitiveToXMLComposer::compose(std::list<sax::Token>& out, const Bool& primitive) const {
	out.push_back(sax::Token(alib::Names::PRIMITIVE_BOOL, sax::Token::TokenType::START_ELEMENT));
	if(primitive.getData())
		out.push_back(sax::Token("true", sax::Token::TokenType::END_ELEMENT));
	else
		out.push_back(sax::Token("false", sax::Token::TokenType::END_ELEMENT));
	out.push_back(sax::Token(alib::Names::PRIMITIVE_BOOL, sax::Token::TokenType::END_ELEMENT));
}

void PrimitiveToXMLComposer::compose(std::list<sax::Token>& out, const String& primitive) const {
	out.push_back(sax::Token(alib::Names::PRIMITIVE_STRING, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(primitive.getData(), sax::Token::TokenType::CHARACTER));
	out.push_back(sax::Token(alib::Names::PRIMITIVE_STRING, sax::Token::TokenType::END_ELEMENT));
}

void PrimitiveToXMLComposer::compose(std::list<sax::Token>& out, int primitive) const {
	out.push_back(sax::Token(alib::Names::PRIMITIVE_INTEGER, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(std::itos(primitive), sax::Token::TokenType::CHARACTER));
	out.push_back(sax::Token(alib::Names::PRIMITIVE_INTEGER, sax::Token::TokenType::END_ELEMENT));
}

void PrimitiveToXMLComposer::compose(std::list<sax::Token>& out, char primitive) const {
	out.push_back(sax::Token(alib::Names::PRIMITIVE_CHARACTER, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(std::string(1, primitive), sax::Token::TokenType::CHARACTER));
	out.push_back(sax::Token(alib::Names::PRIMITIVE_CHARACTER, sax::Token::TokenType::END_ELEMENT));
}

void PrimitiveToXMLComposer::compose(std::list<sax::Token>& out, const std::string& primitive) const {
	out.push_back(sax::Token(alib::Names::PRIMITIVE_STRING, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(primitive, sax::Token::TokenType::CHARACTER));
	out.push_back(sax::Token(alib::Names::PRIMITIVE_STRING, sax::Token::TokenType::END_ELEMENT));
}

void PrimitiveToXMLComposer::compose(std::list<sax::Token>& out, unsigned primitive) const {
	out.push_back(sax::Token(alib::Names::PRIMITIVE_UNSIGNED, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token(std::utos(primitive), sax::Token::TokenType::CHARACTER));
	out.push_back(sax::Token(alib::Names::PRIMITIVE_UNSIGNED, sax::Token::TokenType::END_ELEMENT));
}

void PrimitiveToXMLComposer::compose(std::list<sax::Token>& out, bool primitive) const {
	out.push_back(sax::Token(alib::Names::PRIMITIVE_BOOL, sax::Token::TokenType::START_ELEMENT));
	if(primitive)
		out.push_back(sax::Token("true", sax::Token::TokenType::END_ELEMENT));
	else
		out.push_back(sax::Token("false", sax::Token::TokenType::END_ELEMENT));
	out.push_back(sax::Token(alib::Names::PRIMITIVE_BOOL, sax::Token::TokenType::END_ELEMENT));
}

} /* namespace primitive */
