/*
 * StringLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_STRING_H_
#define PRIMITIVE_STRING_H_

#include <string>
#include <ostream>

#include "PrimitiveBase.h"

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class String : public std::acceptor<String, VisitablePrimitiveBase, std::acceptor<String, alib::VisitableObjectBase, PrimitiveBase> > {
protected:
	std::string data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit String(const std::string& data);

	explicit String(std::string&& data);

	virtual PrimitiveBase* clone() const;

	virtual PrimitiveBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const std::string& getData() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const String& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId<String>();
	}
};

} /* namespace primitive */

namespace std {

template<>
struct compare<primitive::String> {
	int operator()(const primitive::String& first, const primitive::String& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* PRIMITIVE_STRING_H_ */

