/*
 * Unsigned.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_UNSIGNED_H_
#define PRIMITIVE_UNSIGNED_H_

#include <ostream>

#include "PrimitiveBase.h"

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class Unsigned : public std::acceptor<Unsigned, VisitablePrimitiveBase, std::acceptor<Unsigned, alib::VisitableObjectBase, PrimitiveBase> > {
protected:
	unsigned data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit Unsigned(unsigned data);

	virtual PrimitiveBase* clone() const;

	virtual PrimitiveBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	unsigned getData() const;

	virtual int compare(const ObjectBase& other) const {
		return -other.compare(*this);
	}

	virtual int compare(const Unsigned& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual operator std::string() const;

	virtual int selfTypeId() const {
		return typeId<Unsigned>();
	}
};

} /* namespace primitive */

namespace std {

template<>
struct compare<primitive::Unsigned> {
	int operator()(const primitive::Unsigned& first, const primitive::Unsigned& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* PRIMITIVE_UNSIGNED_H_ */
