/*
 * Epsilon.cpp
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#include "Epsilon.h"
#include <sstream>
#include <typeinfo>

#include "LinearString.h"
#include "CyclicString.h"

namespace string {

StringBase* Epsilon::clone() const {
	return new Epsilon(*this);
}

StringBase* Epsilon::plunder() && {
	return new Epsilon(std::move(*this));
}

bool Epsilon::removeSymbolFromAlphabet(const alphabet::Symbol & symbol) {
	return alphabet.erase(symbol);
}

std::vector<alphabet::Symbol> Epsilon::getContent() const {
	return {};
}

bool Epsilon::isEmpty() const {
	return true;
}

int Epsilon::compare(const Epsilon& other) const {
	std::compare<std::set<alphabet::Symbol>> comp;
	return comp(alphabet, other.getAlphabet());
}

int Epsilon::compare(const LinearString& other) const {
	if(other.isEmpty()) {
		std::compare<std::set<alphabet::Symbol>> comp;
		return comp(alphabet, other.getAlphabet());
	}

	if(this->selfTypeId() < typeId<LinearString>())
		return -1;
	else
		return 1;
}

int Epsilon::compare(const CyclicString& other) const {
	if(other.isEmpty()) {
		std::compare<std::set<alphabet::Symbol>> comp;
		return comp(alphabet, other.getAlphabet());
	}

	if(this->selfTypeId() < typeId<CyclicString>())
		return -1;
	else
		return 1;
}

void Epsilon::operator>>(std::ostream& out) const {
	out << "(Epsilon)";
}

Epsilon::operator std::string() const {
	return "E";
}

Epsilon Epsilon::EPSILON = Epsilon();

} /* namespace string */
