/*
 * StringFromStringParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "StringFromStringParser.h"
#include "../exception/AlibException.h"
#include "Epsilon.h"
#include "LinearString.h"
#include "CyclicString.h"
#include <vector>

#include "../StringApi.hpp"

namespace string {

String StringFromStringParser::parseString(std::istream& input) const {
	return parseString(input, std::set<FEATURES>({FEATURES::LINEAR, FEATURES::CYCLIC, FEATURES::EPSILON}));
}

String StringFromStringParser::parseString(std::istream& input, const std::set<FEATURES>& features) const {
	StringFromStringLexer::Token token = m_StringLexer.next(input);
	if(token.type == StringFromStringLexer::TokenType::EPSILON) {
		if(!features.count(FEATURES::EPSILON)) throw exception::AlibException();
		return String(Epsilon());
	} else if(token.type == StringFromStringLexer::TokenType::LESS) {
		if(!features.count(FEATURES::CYCLIC)) throw exception::AlibException();
		std::vector<alphabet::Symbol> data = parseContent(input);
		token = m_StringLexer.next(input);
		if(token.type == StringFromStringLexer::TokenType::GREATER) {
			return String(CyclicString(data));
		} else {
			throw exception::AlibException("Invalid cyclic string terminating character");
		}
	} else if(token.type == StringFromStringLexer::TokenType::QUOTE) {
		if(!features.count(FEATURES::LINEAR)) throw exception::AlibException();
		std::vector<alphabet::Symbol> data = parseContent(input);
		token = m_StringLexer.next(input);
		if(token.type == StringFromStringLexer::TokenType::QUOTE) {
			return String(LinearString(data));
		} else {
			throw exception::AlibException("Invalid liner string terminating character");
		}
	} else {
		throw exception::AlibException();
	}
}

std::vector<alphabet::Symbol> StringFromStringParser::parseContent(std::istream& input) const {
	std::vector<alphabet::Symbol> data;
	StringFromStringLexer::Token token = m_StringLexer.next(input);
	while(token.type != StringFromStringLexer::TokenType::GREATER && token.type != StringFromStringLexer::TokenType::QUOTE) {
		m_StringLexer.putback(input, token);
		data.push_back(alib::stringApi<alphabet::Symbol>::parse(input));

		token = m_StringLexer.next(input);
	}
	m_StringLexer.putback(input, token);
	return data;
}

} /* namespace string */
