/*
 * StringToXMLComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "StringToXMLComposer.h"
#include "CyclicString.h"
#include "LinearString.h"
#include "Epsilon.h"

#include "../XmlApi.hpp"

namespace string {

void StringToXMLComposer::compose(std::list<sax::Token>& out, const String& string) const {
	string.getData().Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
}

void StringToXMLComposer::compose(std::list<sax::Token>& out, const StringBase& string) const {
	string.Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
}

void StringToXMLComposer::compose(std::list<sax::Token>& out, const LinearString& string) const {
	out.push_back(sax::Token(alib::Names::STRING_LINEAR_STRING, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token("alphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol: string.getAlphabet()) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.push_back(sax::Token("alphabet", sax::Token::TokenType::END_ELEMENT));

	out.push_back(sax::Token("content", sax::Token::TokenType::START_ELEMENT));
	for(const auto& symbol : string.getContent()) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.push_back(sax::Token("content", sax::Token::TokenType::END_ELEMENT));
	out.push_back(sax::Token(alib::Names::STRING_LINEAR_STRING, sax::Token::TokenType::END_ELEMENT));
}

void StringToXMLComposer::compose(std::list<sax::Token>& out, const CyclicString& string) const {
	out.push_back(sax::Token(alib::Names::STRING_CYCLIC_STRING, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token("alphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol: string.getAlphabet()) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.push_back(sax::Token("alphabet", sax::Token::TokenType::END_ELEMENT));

	out.push_back(sax::Token("content", sax::Token::TokenType::START_ELEMENT));
	for(const auto& symbol : string.getContent()) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.push_back(sax::Token("content", sax::Token::TokenType::END_ELEMENT));
	out.push_back(sax::Token(alib::Names::STRING_CYCLIC_STRING, sax::Token::TokenType::END_ELEMENT));
}

void StringToXMLComposer::compose(std::list<sax::Token>& out, const Epsilon& string) const {
	out.push_back(sax::Token(alib::Names::STRING_EPSILON, sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token("alphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol: string.getAlphabet()) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.push_back(sax::Token("alphabet", sax::Token::TokenType::END_ELEMENT));
	out.push_back(sax::Token(alib::Names::STRING_EPSILON, sax::Token::TokenType::END_ELEMENT));
}

} /* namespace string */
